/*
 * @(#)Introduction.java	1.16 01/03/19 13:54:27
 *
 * Copyright (c) 2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 *  Another picture
 */

import java.io.File;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Color;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.border.*;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;



public class Introduction extends JPanel {

    private PlanarImage source = null;

    public Introduction(String filename) {
        File f = new File(filename);

        if ( f.exists() && f.canRead() ) {
            source = JAI.create("fileload", filename);
        } else {
            return;
        }

        ImageDisplay canvas = new ImageDisplay(source);
        canvas.setBackground(Color.blue);
        canvas.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));

        canvas.setBorder(new CompoundBorder(
                            new EtchedBorder(),
                            new LineBorder(Color.gray, 20)
                        ) );

        Font font = new Font("SansSerif", Font.BOLD, 12);
        JLabel title = new JLabel(" Use left mouse button to adjust contrast, middle to reset.");
        title.setFont(font);
        title.setLocation(0, 32);

        setOpaque(true);
        setLayout(new BorderLayout());
        setBackground(Color.white);

        // add window-level widget
        int borderWidth = 1;
        int width  = 128 + 2*borderWidth;
        int height = 128 + 2*borderWidth;

        Contrast contrast = new Contrast(source,
                                         width,
                                         height);

        // these next 2 calls are order dependent!!!
        contrast.setDisplay(canvas);
        contrast.setSliderLimits(0.0, 256.0, 0.0, 2.0*256.0);
        contrast.setSliderLocation(width/2, height/2);

        contrast.setBorder(new LineBorder(Color.white,
                                          borderWidth));

        contrast.setBackground(Color.black);
        contrast.setSliderColor(Color.green);
        contrast.setSliderOpaque(true);

        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(new JLabel(" Window-Level"),
                             BorderLayout.NORTH);

        panel.add(contrast);
        canvas.add(panel, BorderLayout.NORTH);

        add(title,  BorderLayout.NORTH);
        add(canvas, BorderLayout.CENTER);

        // set and label contrast widget axes
        contrast.setLayout(new BorderLayout());
        JLabel jx1 = new JLabel("0");
        JLabel jx2 = new JLabel("L");
        JLabel jy1 = new JLabel("0");
        JLabel jy2 = new JLabel("W");

        jx1.setForeground(Color.white);
        jx2.setForeground(Color.white);
        jy1.setForeground(Color.white);
        jy2.setForeground(Color.white);

        JPanel ptop = new JPanel();
        ptop.setBackground(Color.black);
        ptop.setLayout(new FlowLayout(FlowLayout.CENTER));
        ptop.add(jy1);

        JPanel pbot = new JPanel();
        pbot.setBackground(Color.black);
        pbot.setLayout(new FlowLayout(FlowLayout.CENTER));
        pbot.add(jy2);

        contrast.add(jx1, BorderLayout.WEST);
        contrast.add(jx2, BorderLayout.EAST);
        contrast.add(ptop, BorderLayout.NORTH);
        contrast.add(pbot, BorderLayout.SOUTH);
    }
}
