/*
 * @(#)RenderableDemo.java	1.8 01/03/19 13:54:28
 *
 * Copyright (c) 2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.File;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.media.jai.*;



public class RenderableDemo extends JPanel
                            implements ActionListener {

    private PlanarImage source = null;
    private ImageDisplay canvas = null;
    private boolean state = false;

    public RenderableDemo(String filename) {
        File f = new File(filename);

        if ( f.exists() && f.canRead() ) {
            source = JAI.create("fileload", filename);
        } else {
            return;
        }

        ImageDisplay display = new ImageDisplay(source);
        display.setLayout(new FlowLayout(FlowLayout.RIGHT, 2, 2));

        Panner panner = new Panner(display, source, 128);
        panner.setBackground(Color.white);
        panner.setBorder(new EtchedBorder());
        display.add(panner);

        ParameterBlock pb = new ParameterBlock();
        pb.addSource(source);
        pb.add(null).add(null).add(null).add(null).add(null);

        RenderableImage ren = JAI.createRenderable("renderable", pb);

        pb = new ParameterBlock();
        pb.addSource(ren);

        RenderableImage inv = JAI.createRenderable("invert", pb);

        int w = source.getWidth()/4;
        int h = source.getHeight()/4;

        PlanarImage dst = (PlanarImage)inv.createScaledRendering(w, h, null);

        canvas = new ImageDisplay(dst);
        canvas.setBackground(Color.blue);

        canvas.setBorder(new CompoundBorder(
                                           new EtchedBorder(),
                                           new LineBorder(Color.gray, 5)
                                           ) );

        JPanel yap = new JPanel();  // yet another panel
        yap.setLayout(new BorderLayout());
        yap.add(new JLabel("Destination"), BorderLayout.NORTH);

        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 6));
        panel.add(canvas);

        // space the button
        JLabel spacer = new JLabel("      ");
        panel.add(spacer);

        JButton button = new JButton("Invert/Scale");
        button.setMargin(new Insets(2, 5, 2, 5));
        panel.add(button);

        button.addActionListener(this);

        Font font = new Font("SansSerif", Font.BOLD, 12);
        JLabel title = new JLabel(" Renderable Imaging Source");
        title.setFont(font);
        title.setLocation(0, 32);

        setOpaque(true);
        setLayout(new BorderLayout());
        setBackground(Color.white);

        yap.add(panel,   BorderLayout.CENTER);
        add(title,   BorderLayout.NORTH);
        add(display, BorderLayout.CENTER);
        add(yap,   BorderLayout.SOUTH);
    }

    public void actionPerformed(ActionEvent e) {
        RenderableImage inv;

        ParameterBlock pb = new ParameterBlock();
        pb.addSource(source);
        pb.add(null).add(null).add(null).add(null).add(null);

        RenderableImage ren = JAI.createRenderable("renderable", pb);

        pb = new ParameterBlock();
        pb.addSource(ren);

        if ( state == false ) {
            inv = ren;
            state = true;
        } else {
            inv = JAI.createRenderable("invert", pb);
            state = false;
        }

        int w = source.getWidth()/4;
        int h = source.getHeight()/4;

        PlanarImage dst = (PlanarImage)inv.createScaledRendering(w, h, null);
        canvas.set(dst);
    }
}
