/*
 * @(#)RenderableScale.java	1.11 01/03/19 13:54:28
 *
 * Copyright (c) 2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.media.jai.*;


public class RenderableScale extends JPanel
                             implements ChangeListener {

    private RenderableDisplay canvas = null;
    private final int width  = 185;
    private final int height = 173;

    public RenderableScale(String base) {

        Vector sources = new Vector();

        for (int i = 3; i >= 0; i-- ) {
            int ext = i + 1;
            String file = base + ext + ".jpg";
            RenderedImage im = JAI.create("fileload", file);
            sources.addElement(im);
        }

        RenderableImage r = new MultiResolutionRenderableImage(sources,
                                                               0.0F,
                                                               0.0F,
                                                               1.0F);

        setLayout(new BorderLayout());

        JLabel label = new JLabel("Scalable Image");
        add(label, BorderLayout.NORTH);

        canvas = new RenderableDisplay(r);
        add(canvas, BorderLayout.CENTER);
        setVisible(true);

        JSlider slider = new JSlider(JSlider.HORIZONTAL, 1, 30, 30);
        slider.addChangeListener(this);

        JPanel borderedPanel = new JPanel();
        borderedPanel.setLayout(new BorderLayout());
        borderedPanel.setBorder(BorderFactory.createTitledBorder("Zoom"));
        borderedPanel.add(slider, BorderLayout.NORTH);

        add(borderedPanel, BorderLayout.SOUTH);

        canvas.setSize(new Dimension(18, 17));
    }

    public final void stateChanged(ChangeEvent e) {
        JSlider slider = (JSlider) e.getSource();
        int scale = slider.getValue();

        int nw = (scale * width) / 10;
        int nh = (scale * height) / 10;

        canvas.setSize(new Dimension(nw, nh));
    }
}
